#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <qlistwidget.h>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    /* Get a handle to the USB device */
    USBDevice = new CCyUSBDevice((HANDLE)0, CYUSBDRV_GUID, true);

    /* Get references to all UI widgets we'll be using */
    listWidget = ui->centralWidget->findChild<QListWidget*>("listWidget");
    comboBox = ui->centralWidget->findChild<QComboBox*>("comboBox");
    pushButton = ui->centralWidget->findChild<QPushButton*>("pushButton");
    plainTextEdit = ui->centralWidget->findChild<QPlainTextEdit*>("plainTextEdit");

    /* Update the list view once */
    UpdateListView(listWidget);

    /* Disable the "Transfer" button before any device was selected */
    pushButton->setEnabled(false);

    /* Attach handlers for all UI widgets */
    connect(listWidget, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(listItemClicked(QListWidgetItem*)));
    connect(comboBox,SIGNAL(currentIndexChanged(const QString&)), this, SLOT(epSelectionChanged(const QString&)));
    connect(pushButton, SIGNAL(clicked()), this, SLOT(transferButtonClicked()));
}

void MainWindow::transferButtonClicked(void)
{
    /* No support for Control tranfers now */
    if((EndPoint->Address & 0x7F) == 0x00)
    {
        plainTextEdit->insertPlainText("Control transfers not supported at the moment\n");
        return;
    }

    /* No support for Isoc tranfers now */
    if(EndPoint->Attributes == 1)
    {
        plainTextEdit->insertPlainText("Isochronous transfers not supported at the moment\n");
        return;
    }

    UCHAR * xferBuffer[EndPoint->MaxPktSize];
    LONG xferLength = EndPoint->MaxPktSize;

    /* Transfer 1 packet and check if that succeeds */
    if(EndPoint->XferData((PUCHAR)xferBuffer, xferLength, (CCyIsoPktInfo*)0, (BOOL)true))
        plainTextEdit->insertPlainText(QString::number(xferLength) + " bytes transfer was successful\n");
    else
        plainTextEdit->insertPlainText(QString::number(xferLength) + " bytes transfer failed\n");
}

void MainWindow::epSelectionChanged(const QString& epString)
{
    bool OK;

    /* Sometimes we get a null string. Ignore those. */
    if(epString == "")
        return;

    /* Find out the new endpoint's number */
    const unsigned int epNum = epString.split(" ").at(2).toUInt(&OK, 16);
    EndPoint = USBDevice->EndPointOf(epNum);
    pushButton->setEnabled(true);
}

void MainWindow::listItemClicked(QListWidgetItem* item)
{
    /* Get the index of the device selected by the user and open it */
    int index = this->listWidget->currentRow();
    USBDevice->Open(index);

    this->comboBox->clear();

    for(int i = 0; i < USBDevice->EndPointCount(); i++)
    {
        /* Add the corresponding endpoint to our combo list */
        UCHAR epAddr = USBDevice->EndPoints[i]->Address;
        if(epAddr & 0x80)
            this->comboBox->addItem(QString("IN Endpoint 0x") + QString::number(epAddr, 16));
        else
            this->comboBox->addItem(QString("OUT Endpoint 0x") + QString::number(epAddr, 16));
    }
}

void MainWindow::UpdateListView(QListWidget *listWidget)
{
    int numDevices = USBDevice->DeviceCount();
    listWidget->clear();

    /* Find all devices and add them to our list view */
    for(int i = 0; i < numDevices; i++)
    {
        USBDevice->Open(i);
        listWidget->addItem(USBDevice->FriendlyName);
        USBDevice->Close();
    }
}

/* The destructor that's called when the UI is taken down */
MainWindow::~MainWindow()
{
    delete ui;
}
